<?php
//
// Manage config section of Nagios XI admin panel
// Copyright (c) 2008-2015 Nagios Enterprises, LLC. All rights reserved.
//
// $Id$

require_once(dirname(__FILE__) . '/../includes/common.inc.php');
require_once(dirname(__FILE__) . '/../includes/utils-timezones.inc.php');

// Initialization stuff
pre_init();
init_session();

// Grab GET or POST variables 
grab_request_vars();

// Check prereqs and authentication
check_prereqs();
check_authentication(false);

// Only admins can access this page
if (is_admin() == false) {
    echo $lstr['NotAuthorizedErrorText'];
    exit();
}

route_request();

function route_request()
{
    global $request;
    if (isset($request['update'])) {
        do_update_options();
    } else {
        show_options();
    }
}

/**
 * @param bool   $error
 * @param string $msg
 */
function show_options($error = false, $msg = "", $reset = false)
{
    global $request;
    global $lstr;

    $url = get_option('url');
    if (have_value($url) == false) {
        $url = get_base_url();
    }

    if (isset($_SESSION['msg'])) {
        $msg = $_SESSION['msg'];
        if (!$reset) {
            unset($_SESSION['msg']);
        }
    }

    // Get options
    $url = grab_request_var("url", $url);
    $external_url = grab_request_var("external_url", get_option('external_url'));
    $admin_name = grab_request_var("admin_name", get_option('admin_name'));
    $admin_email = grab_request_var("admin_email", get_option('admin_email'));
    $language = grab_request_var("defaultLanguage", get_option('default_language'));
    $date_format = grab_request_var("defaultDateFormat", get_option('default_date_format'));
    $number_format = grab_request_var("defaultNumberFormat", intval(get_option('default_number_format')));

    // System setting for new XI theme
    $theme = grab_request_var("theme", get_option('theme', 'xi5'));
    $hc_theme = grab_request_var("hc_theme", get_option('default_highcharts_theme'));
    $highchart_scale = grab_request_var("highchart_scale", get_option('highchart_scale'));
    $perfdata_theme = grab_request_var("perfdata_theme", get_option('perfdata_theme'));
    $wc_display = grab_request_var("wc_display", get_option('wc_display'));

    // Enable writing audit log to file - bool
    $auditlogfile = grab_request_var("auditlogfile", get_option('auditlogfile'));

    // CCM auto-login integration
    $separate_ccm_login = grab_request_var("separate_ccm_login", get_option('separate_ccm_login', 0));

    // Config Timezone
    $cfg_timezone = grab_request_var("timezone", get_option('timezone'));

    // Acknowledgement defaults
    $adefault_sticky_acknowledgment = grab_request_var('adefault_sticky_acknowledgment', get_option('adefault_sticky_acknowledgment', 1));
    $adefault_send_notification = grab_request_var('adefault_send_notification', get_option('adefault_send_notification', 1));
    $adefault_persistent_comment = grab_request_var('adefault_persistent_comment', get_option('adefault_persistent_comment'));

    // Fuse key
    $fusekey = get_option('fusekey');
    if (empty($fusekey)) {
        $fusekey = strtoupper(md5(uniqid()));
        set_option('fusekey', $fusekey);
    }

    if (substr($url, -1) != '/') { $url .= '/'; }
    if (substr($external_url, -1) != '/' && !empty($external_url)) { $external_url .= '/'; }

    // If perfdata theme doesn't exist we should set it to 1
    if (get_option('perfdata_theme') == NULL) {
        $perfdata_theme = 1;
    }

    // Set defaults for admin name/email
    if ($admin_name == "") {
        $admin_name = "Nagios XI Admin";
    }
    if ($admin_email == "") {
        $admin_email = "root@localhost";
    }

    // Default to check for updates unless overridden
    $auc = get_option('auto_update_check');
    if ($auc == "") {
        $auc = 1;
    }
    $auto_update_check = grab_request_var("auto_update_check", $auc);
    if ($auto_update_check == "on") {
        $auto_update_check = 1;
    }

    // Allow html in status text?
    $allow_html = grab_request_var('allow_html', get_option('allow_status_html'));

    // Get global variables
    $languages = get_languages();
    $number_formats = get_number_formats();
    $date_formats = get_date_formats();

    // Start actual "System Settings" page output
    do_page_start(array("page_title" => _("System Settings")), true);
    ?>

    <script type="text/javascript">
    <?php if ($reset) { ?>
    window.parent.location.href="<?php echo get_base_url(); ?>admin/?xiwindow=globalconfig.php";
    <?php } ?>
    </script>

    <h1><?php echo _("System Settings"); ?></h1>
    <?php display_message($error, false, $msg); ?>
    <form id="manageOptionsForm" method="post" action="<?php echo encode_form_val($_SERVER['PHP_SELF']); ?>">
        <input type="hidden" name="options" value="1">
        <?php echo get_nagios_session_protector(); ?>
        <input type="hidden" name="update" value="1">

        <script>
        $(document).ready(function() {
            $('#tabs').tabs().show();
        });
        </script>

        <div id="tabs" class="hide">
            <ul>
                <li><a href="#general"><i class="fa fa-cog"></i> <?php echo _('General'); ?></a></li>
                <li><a href="#display"><i class="fa fa-desktop"></i> <?php echo _('Theme & Display'); ?></a></li>
                <li><a href="#defaults"><i class="fa fa-user"></i> <?php echo _('User Defaults'); ?></a></li>
                <li><a href="#integration"><i class="fa fa-key"></i> <?php echo _('Integration'); ?></a></li>
            </ul>
            <div id="general">
                <h5 class="ul"><?php echo _("General Program Settings"); ?></h5>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td class="vt">
                            <label for="urlBox"><?php echo _("Program URL"); ?>:</label>
                        </td>
                        <td>
                            <input type="text" size="45" name="url" id="urlBox" value="<?php echo encode_form_val($url); ?>" class="textfield form-control">
                            <div class="subtext"><?php echo _("The default URL used to access Nagios XI directly from your internal network"); ?>.</div>
                        </td>
                    </tr>
                    <tr>
                        <td class="vt">
                            <label for="externalurlBox"><?php echo _("External URL"); ?>:</label>
                        </td>
                        <td>
                            <input type="text" size="45" name="external_url" id="externalurlBox" value="<?php echo encode_form_val($external_url); ?>" class="textfield form-control">
                            <div class="subtext"><?php echo _("The URL used to access Nagios XI from outside of your internal network (if different than the default above).  If defined, this URL will be referenced in email alerts and generated pdf reports to allow quick access to the XI interface"); ?>.</div>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="adminNameBox"><?php echo _("Administrator Name"); ?>:</label>
                        </td>
                        <td>
                            <input type="text" size="30" name="admin_name" id="adminNameBox" value="<?php echo encode_form_val($admin_name); ?>" class="textfield form-control">
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="adminEmailBox"><?php echo _("Administrator Email"); ?>:</label>
                        </td>
                        <td>
                            <input type="text" size="30" name="admin_email" id="adminEmailBox" value="<?php echo encode_form_val($admin_email); ?>" class="textfield form-control">
                        </td>
                    </tr>
                    <tr>
                        <td></td>
                        <td class="checkbox">
                            <label class="checkbox">
                                <input type="checkbox" class="checkbox" id="autoUpdateCheckBox" name="auto_update_check" <?php echo is_checked($auto_update_check, 1); ?>>
                                <?php echo _("Automatically Check for Updates"); ?>
                                <span>(<a href="<?php echo get_update_check_url(); ?>" target="_blank"><?php echo _("Check Now"); ?></a>)</span>
                            </label>
                        </td>
                    </tr>
                    <tr>
                        <td></td>
                        <td class="checkbox">
                            <label class="checkbox">
                                <input type="checkbox" id="allow_html" name="allow_html" <?php echo is_checked($allow_html); ?>>
                                <?php echo _("Allow HTML Tags in Host/Service Status"); ?>
                            </label>
                        </td>
                    </tr>
                </table>

                <?php
                $current_timezone = get_current_timezone();
                if (!empty($cfg_timezone) && $cfg_timezone != $current_timezone) {
                    $current_timezone = $cfg_timezone;
                }
                $timezones = get_timezones();
                ?>

                <h5 class="ul"><?php echo _("Timezone Settings"); ?></h5>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td><label><?php echo _("Timezone"); ?>:</label></td>
                        <td>
                            <select id="timezone" name="timezone" class="form-control">
                                <?php
                                $set = false;
                                foreach ($timezones as $name => $tz) {
                                    ?>
                                    <option value="<?php echo $tz; ?>"<?php if ($tz == $current_timezone) {
                                        echo "selected";
                                        $set = true;
                                    } ?>><?php echo $name; ?></option>
                                <?php
                                }

                                if (!$set) {
                                    ?>
                                    <option value="<?php echo $current_timezone; ?>"
                                            selected><?php echo $current_timezone; ?></option>
                                <?php
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                </table>

                <h5 class="ul"><?php echo _("CCM Integration Settings"); ?></h5>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td>
                            <label for="sccmlogin"><?php echo _("Separate CCM Login"); ?>:</label>
                        </td>
                        <td class="checkbox">
                            <label>
                                <input type="checkbox" name="separate_ccm_login" id="sccmlogin" value="1" <?php echo is_checked($separate_ccm_login, 1); ?>>
                                <?php echo _("Use separate CCM-only logins for each user. (The old way of CCM authentication)"); ?>
                            </label>
                        </td>
                    </tr>
                </table>

                <h5 class="ul"><?php echo _("Other Settings"); ?></h5>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td>
                            <label for="auditlogfile"><?php echo _("Write Audit Log to file"); ?>:</label>
                        </td>
                        <td class="checkbox">
                            <label>
                                <input type="checkbox" name="auditlogfile" id="auditlogfile" value="1" <?php echo is_checked($auditlogfile, 1); ?>>
                                <?php echo _("When checked all audit log events will be written to "); ?><b>/usr/local/nagiosxi/var/components/auditlog.log</b>
                            </label>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label><?php echo _("Acknowledgement Defaults"); ?>:</label>
                        </td>
                        <td class="checkbox">
                            <label>
                                <input type="checkbox" name="adefault_sticky_acknowledgment" value="1" <?php echo is_checked($adefault_sticky_acknowledgment, 1); ?>> <?php echo _('Sticky Acknowledgement'); ?>
                            </label>
                            <label style="margin-left: 10px;">
                                <input type="checkbox" name="adefault_send_notification" value="1" <?php echo is_checked($adefault_send_notification, 1); ?>> <?php echo _('Send Notification'); ?>
                            </label>
                            <label style="margin-left: 10px;">
                                <input type="checkbox" name="adefault_persistent_comment" value="1" <?php echo is_checked($adefault_persistent_comment, 1); ?>> <?php echo _('Persistent Comment'); ?>
                            </label>
                        </td>
                    </tr>
                </table>
            </div>
            <div id="display">
                <h5 class="ul"><?php echo _("Theme Settings"); ?></h5>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td><label><?php echo _("XI User Interface Theme"); ?>:</label></td>
                        <td>
                            <select id="theme" name="theme" class="form-control">
                                <option value="xi5"<?php if ($theme == 'xi5') { echo " selected"; } ?>><?php echo _("XI 5 - Modern"); ?></option>
                                <option value="xi2014"<?php if ($theme == 'xi2014') { echo " selected"; } ?>><?php echo _("XI 2014"); ?></option>
                                <option value="classic"<?php if ($theme == 'classic') { echo " selected"; } ?>><?php echo _("Classic XI"); ?></option>
                            </select>
                        </td>
                    </tr>
                </table>
                <h5 class="ul"><?php echo _("Display Settings"); ?></h5>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td><label><?php echo _("Highcharts Color Theme"); ?>:</label></td>
                        <td>
                            <select id="hc_theme" name="hc_theme" class="form-control">
                                <option value="default"<?php if ($hc_theme == 'default') { echo " selected"; } ?>><?php echo _("Default (White)"); ?></option>
                                <option value="gray"<?php if ($hc_theme == 'gray') { echo " selected"; } ?>><?php echo _("Classic (Gray)"); ?></option>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td></td>
                        <td class="checkbox">
                            <label class="checkbox">
                                <input type="checkbox" class="checkbox" id="perfdataThemeCheckBox" name="perfdata_theme" value="1" <?php echo is_checked($perfdata_theme, 1); ?>>
                                <?php echo _("Use Highcharts for Perfdata Graphs"); ?>
                            </label>
                        </td>
                    </tr>
                    <tr>
                        <td><label><?php echo _("Scale for Highcharts Graphs"); ?>:</label></td>
                        <td>
                            <select id="highchart_scale" name="highchart_scale" class="form-control">
                                <option value="linear"<?php if ($highchart_scale == 'linear') { echo " selected"; } ?>><?php echo _("Linear"); ?></option>
                                <option value="logarithmic"<?php if ($highchart_scale == 'logarithmic') { echo " selected"; } ?>><?php echo _("Logarithmic"); ?></option>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td><label><?php echo _("Warning/Critical Line Display"); ?>:</label></td>
                        <td>
                            <select id="wc_display" name="wc_display" class="form-control">
                                <option value="0"<?php if ($wc_display == 0) { echo " selected"; } ?>><?php echo _("Off"); ?></option>
                                <option value="1"<?php if ($wc_display == 1) { echo " selected"; } ?>><?php echo _("On"); ?></option>
                            </select>
                        </td>
                    </tr>
                </table>
            </div>
            <div id="defaults">
                <h5 class="ul"><?php echo _("Default User Settings"); ?></h5>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td>
                            <label for="defaultLanguage"><?php echo _("Language"); ?>:</label>
                        </td>
                        <td>
                            <select name="defaultLanguage" class="languageList dropdown form-control">
                                <?php
                                foreach ($languages as $lang => $title) {
                                    echo '<option value="' . $lang . '" ' . is_selected($language, $lang) . '>' . get_language_nicename($title) . '</option>';
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="defaultDateFormat"><?php echo _("Date Format"); ?>:</label>
                        </td>
                        <td>
                            <select name="defaultDateFormat" class="dateformatList dropdown form-control">
                                <?php
                                foreach ($date_formats as $id => $txt) {
                                    echo '<option value="' . $id . '" ' . is_selected($id, $date_format) . '>' . $txt . '</option>';
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                    <tr>
                        <td>
                            <label for="defaultNumberFormat"><?php echo _("Number Format"); ?>:</label>
                        </td>
                        <td>
                            <select name="defaultNumberFormat" class="numberformatList dropdown form-control">
                                <?php
                                foreach ($number_formats as $id => $txt) {
                                    echo '<option value="' . $id . '" ' . is_selected($id, $number_format) . '>' . $txt . '</option>';
                                }
                                ?>
                            </select>
                        </td>
                    </tr>
                </table>
            </div>
            <div id="integration">
                <h5 class="ul"><?php echo _('Nagios Fusion Integration'); ?></h5>
                <p><?php echo _('The fuse key below should be given to your Nagios Fusion 4 server only. The key allows a Fusion 4 instance to connect and integrate with this Nagios XI system.'); ?></p>
                <table class="table table-condensed table-no-border table-auto-width">
                    <tr>
                        <td><label><?php echo _('Fuse Key'); ?>:</label></td>
                        <td><input type="text" name="fusekey" value="<?php echo $fusekey; ?>" class="form-control" style="width: 280px;" readonly></td>
                    </tr>
                </table>
            </div>
        </div>
        
        <div id="formButtons">
            <input type="submit" class="submitbutton btn btn-sm btn-primary" name="updateButton" value="<?php echo _("Update Settings"); ?>" id="updateButton">
            <input type="submit" class="submitbutton btn btn-sm btn-default" name="cancelButton" value="<?php echo _("Cancel"); ?>" id="cancelButton">
        </div>

    </form>
    <?php
    do_page_end(true);
}

// Save the options we just set with the above form
function do_update_options()
{
    global $request;
    global $lstr;

    // User pressed the cancel button
    if (isset($request["cancelButton"])) {
        header("Location: main.php");
    }

    // Check session
    check_nagios_session_protector();

    $errmsg = array();
    $errors = 0;

    // Get values
    $auto_update_check = grab_request_var("auto_update_check", "");
    $auto_update_check = ((have_value($auto_update_check)) ? 1 : 0);
    $admin_name = grab_request_var("admin_name", "");
    $admin_email = grab_request_var("admin_email", "");
    $url = grab_request_var("url", "");
    $external_url = grab_request_var("external_url", "");
    $date_format = grab_request_var("defaultDateFormat", DF_ISO8601);
    $number_format = grab_request_var("defaultNumberFormat", NF_2);
    $language = grab_request_var("defaultLanguage", "");
    $allow_html = grab_request_var('allow_html', false);

    // Theme settings for 2014
    $theme = grab_request_var("theme", "");
    $hc_theme = grab_request_var("hc_theme", "");
    $highchart_scale = grab_request_var("highchart_scale", "");
    $perfdata_theme = grab_request_var("perfdata_theme", 0);
    $wc_display = grab_request_var("wc_display", 0);

    $auditlogfile = grab_request_var("auditlogfile", 0);

    // Grab the CCM auto-login integration
    $separate_ccm_login = grab_request_var("separate_ccm_login", 0);

    // Get the timezone
    $new_timezone = grab_request_var("timezone", "");

    // Acknowledgement defaults
    $adefault_sticky_acknowledgment = grab_request_var('adefault_sticky_acknowledgment', 0);
    $adefault_send_notification = grab_request_var('adefault_send_notification', 0);
    $adefault_persistent_comment = grab_request_var('adefault_persistent_comment', 0);

    // Make sure we have requirements
    if (in_demo_mode() == true)
        $errmsg[$errors++] = $lstr['DemoModeChangeError'];
    if (have_value($admin_name) == false)
        $errmsg[$errors++] = $lstr["NoAdminNameError"];
    if (have_value($admin_email) == false)
        $errmsg[$errors++] = $lstr["NoAdminEmailError"];
    else if (!valid_email($admin_email))
        $errmsg[$errors++] = $lstr["InvalidAdminEmailError"];
    if (have_value($url) == false)
        $errmsg[$errors++] = $lstr['BlankURLError'];
    else if (!valid_url($url))
        $errmsg[$errors++] = $lstr['InvalidURLError'];
    if (have_value($language) == false)
        $errmsg[$errors++] = $lstr['BlankDefaultLanguageError'];

    // Handle errors
    if ($errors > 0) {
        show_options(true, $errmsg);
    }

    if (substr($url, -1) != '/') { $url .= '/'; }
    if (substr($external_url, -1) != '/' && !empty($external_url)) { $external_url .= '/'; }

    // Update options
    set_option("admin_name", $admin_name);
    set_option("admin_email", $admin_email);
    set_option("url", $url);
    set_option("external_url", $external_url);
    set_option("default_language", $language);
    set_language($language);
    set_option("auto_update_check", $auto_update_check);
    set_option("default_date_format", $date_format);
    set_option("default_number_format", $number_format);
    set_option('allow_status_html', $allow_html);

    set_option('adefault_sticky_acknowledgment', $adefault_sticky_acknowledgment);
    set_option('adefault_send_notification', $adefault_send_notification);
    set_option('adefault_persistent_comment', $adefault_persistent_comment);

    // Set theme options for 2014
    $old_theme = get_option('theme', '');
    set_option("theme", $theme);
    set_option("default_highcharts_theme", $hc_theme);
    set_option("highchart_scale", $highchart_scale);
    set_option("perfdata_theme", $perfdata_theme);
    set_option("wc_display", $wc_display);

    // check if auditlog write checkbox changed- then log action
    $auditlog_toggle = get_option("auditlogfile", 0);
    $auditlog_path = '/usr/local/nagiosxi/var/components/auditlog.log';

    if ($auditlog_toggle != $auditlogfile) {
        if ($auditlogfile) {
            set_option("auditlogfile", $auditlogfile);

            // create log file set perms
            exec("touch {$auditlog_path}", $auditlog_arr, $auditlog_create);
            exec("chown apache:nagios {$auditlog_path}");
            exec("chmod ug+wx {$auditlog_path}");

            send_to_audit_log("Nagios XI auditlog: writing to file enabled.", AUDITLOGTYPE_CHANGE);
        } else {
            send_to_audit_log("Nagios XI auditlog: writing to file disabled.", AUDITLOGTYPE_CHANGE);
            set_option("auditlogfile", $auditlogfile);
        }
    }

    // Set new CCM integration option
    set_option("separate_ccm_login", $separate_ccm_login);

    // Set the timezone (so we can update it when page reloads)
    set_option("timezone", $new_timezone);

    // Mark that system settings were configured
    set_option("system_settings_configured", 1);

    // Log it
    send_to_audit_log("User updated global program settings", AUDITLOGTYPE_CHANGE);

    // Update the timezone if we need to!
    $current_timezone = get_current_timezone();
    if (!empty($new_timezone) && $current_timezone != $new_timezone) {
        submit_command(COMMAND_CHANGE_TIMEZONE, $new_timezone);
    }

    $reset = false;
    $user_theme = get_user_meta(0, "theme");
    if ($old_theme != $theme && $user_theme == '') {
        $reset = true;
        $_SESSION['msg'] = _('Settings updated. (Screen may flash once while UI theme changes)');
    }

    show_options(false, _('Settings updated.'), $reset);
}
