/* General Overlay Functions */

$(document).ready(function () {

    // On window resize, re-draw any visible overlays
    $(window).resize(function() {
        $('.overlay:visible').each(function(k, v) {
            overlay($(v).attr('id'));
        });
    });

    // Activate list filter text input
    $('#selHosts').filterByText($('#filterHosts'), true);
    $('#selServices').filterByText($('#filterServices'), true);
    $('#selHosttemplates').filterByText($('#filterHosttemplates'), true);
    $('#selServicetemplates').filterByText($('#filterServicetemplates'), true);

    // Activate dropdown search
    $('#timeperiods').searchable({maxMultiMatch: 9999});

    $('#option_list').change(function() {
        var type = $('#option_list option:selected').data('type');
        var form_html = '';

        $('#timeperiod_config_option').hide();
        $('#inner_config_option').show();

        switch (type) {

            case 'field':
                var uom = $('#option_list option:selected').data('uom');
                form_html += '<label>Value: <input type="text" size="2" value="" class="form-control" name="field_value"> '+uom+'</label>';
                break;

            case 'oosn':
                form_html += '<label><input type="radio" value="1" name="oosn_value"> on</label>';
                form_html += '<label><input type="radio" value="0" name="oosn_value"> off</label>';
                form_html += '<label><input type="radio" value="2" name="oosn_value"> skip</label>';
                form_html += '<label><input type="radio" value="3" name="oosn_value"> null</label>';
                break;

            case 'dou':
                var input_type = 'checkbox';
                if ($('#option_list option:selected').data('r') == '1') {
                    input_type = 'radio';
                }
                form_html += '<div><label>Hosts:</label>';
                form_html += '<label><input type="'+input_type+'" value="d" name="host_opts_value[]"> d</label>';
                form_html += '<label><input type="'+input_type+'" value="o" name="host_opts_value[]"> o</label>';
                form_html += '<label><input type="'+input_type+'" value="u" name="host_opts_value[]"> u</label></div>';
                form_html += '<div><label>Services:</label>';
                form_html += '<label><input type="'+input_type+'" value="w" name="service_opts_value[]"> w</label>';
                form_html += '<label><input type="'+input_type+'" value="c" name="service_opts_value[]"> c</label>';
                form_html += '<label><input type="'+input_type+'" value="o" name="service_opts_value[]"> o</label>';
                form_html += '<label><input type="'+input_type+'" value="u" name="service_opts_value[]"> u</label></div>';
                break;

            case 'nopts':
                form_html += '<div><label>Hosts:</label>';
                form_html += '<label><input type="checkbox" value="d" name="host_opts_value[]"> d</label>';
                form_html += '<label><input type="checkbox" value="u" name="host_opts_value[]"> u</label>';
                form_html += '<label><input type="checkbox" value="r" name="host_opts_value[]"> r</label>';
                form_html += '<label><input type="checkbox" value="f" name="host_opts_value[]"> f</label>';
                form_html += '<label><input type="checkbox" value="s" name="host_opts_value[]"> s</label></div>';
                form_html += '<div><label>Services:</label>';
                form_html += '<label><input type="checkbox" value="w" name="service_opts_value[]"> w</label>';
                form_html += '<label><input type="checkbox" value="c" name="service_opts_value[]"> c</label>';
                form_html += '<label><input type="checkbox" value="u" name="service_opts_value[]"> u</label>';
                form_html += '<label><input type="checkbox" value="r" name="service_opts_value[]"> r</label>';
                form_html += '<label><input type="checkbox" value="f" name="service_opts_value[]"> f</label>';
                form_html += '<label><input type="checkbox" value="s" name="service_opts_value[]"> s</label></div>';
                break;

            case 'dd':
                if (!$('#timeperiod_config_option').is(':visible')) {
                    $('#timeperiod_config_option').show();
                    $('#inner_config_option').hide();
                }
                break;

            default:
                form_html = 'Not implemented yet.';
                break;

        }

         $('#inner_config_option').html(form_html);
    });

    // When command selection changes
    $('#commands').change(function() {
        var id = $('#commands option:selected').val();
        if (id != 'blank' && id != '') {
            $('#fullcommand').html(command_list[id]);
            $('#command-box').show();
        } else {
            $('#command-box').hide();
        }
        if (id == 'blank') {
            $('.arg-box input').attr('disabled', true);
        } else {
            $('.arg-box input').attr('disabled', false);
        }
    });

    // Host/Service type selection in modify templates
    $('.hs-template-select').click(function() {
        $('#change_templates_selector').hide();
        var type = $(this).data('type');
        $('#bulk_change_templates').show();
        if (type == 'host') {
            $('#templates-hosts').show();
        } else if (type == 'service') {
            $('#templates-services').show();
        }
    });

});

function overlay(div)
{
    var ID = '#'+div;

    // Check if overlay actually exists 
    if ($(ID).html() == null) {
        alert('Undefined overlay');
        return;
    }

    // Cover the page with a opaque hidden div...
    var height = $(window).height() + 20;
    var width = $(window).width() + 20;
    $('#screen-overlay').css("width", width)
                        .css("height", height)
                        .css("background-color", "#FFF")
                        .css("position", "fixed")
                        .css("left", "-10px")
                        .css("top", "-10px")
                        .css("opacity", "0.85")
                        .show();
    
    // Make the overlay width much bigger if it's a small screen
    if ($(window).width() < 1024) {
        var overlay_width = width-(width*0.10);
    } else {
        var overlay_width = width-(width*0.40);
    }
    var overlay_height = height-(height*0.20);

    // Set defaults (min and max)
    if (overlay_width < 400) { overlay_width = 400; }
    if (overlay_height > 500) { overlay_height = 500; }

    // Generate the left and top margins
    var top = ($(window).height() - overlay_height-46) / 2;
    var left = ($(window).width() - overlay_width-46) / 2;
    if (top > 100) {
        top = 100;
    }

    // Display the selection overlay
    $(ID).css('width', overlay_width)
         .css('left', left+"px")
         .css('top', top+"px")
         .css('position', 'absolute')
         .css('height', overlay_height)
         .show();

    // Resize selection box to be most of the height (use as much real estate as possible)
    var select_height = overlay_height - $(ID).find('.listDiv label').outerHeight() - $(ID).find('.overlay-left-bottom').outerHeight() - 20;
    $(ID).find('.lists').css("height", select_height);
}

// Remove the overlay
function killOverlay(id) {
    $('#screen-overlay').hide();
    ID = '#'+id;
    $(ID).hide();
}

// Hide all lists that can be toggled
function hide() {
    $(".hidden").hide();
}

// Unique identifier for added <tr> rows
var unique = 0;

// Control array for transferMembers() 
var uniqueIDs = [];

///////////////////// transferMembers()  ///////////////////////////////////////// 
//used to select parents, hostgroups, and other items that may have multiple values
//inputDiv - the ID of the select list to pull selected items from 
//outputDiv - the table that the selected items are being added to ->(hidden inputs will be added for the values)
//postArray - the php $_POST array value that these items will be added to 
function transferMembers(inputDiv, outputDiv, postArray, afterLoad)
{
    var titles = []; //display titles for select options
    var values = []; //option values
    var ids = []; //array of id's

    var input = '#' + inputDiv;
    $(input + ' :selected').each(function (i, selected) {
        titles[i] = $(selected).text();
        values[i] = $(selected).val();
        ids[i] = $(selected).attr('id'); //capture item id
    });

    for (i = 0; i < ids.length; i++) {
        // Create data id that ties to option
        unique++;
        var thisID = ids[i];

        if (postArray == 'hostgroups') {
            // Disable all hosts that we just selected
            $.post('ajaxreqs.php', { hostgroup: values[i], cmd: 'gethostsinhostgroup' }, function(data) {
                $.each(data, function(k, v) {
                    $('#tblHosts .outputTableData:contains("'+v+'")').parents('tr').find('.remove a').trigger('click');
                    $('#selHosts').find('option:contains("'+v+'")').attr('disabled', 'disabled').removeAttr('selected');
                });
            }, 'json');
        } else if (postArray == 'servicegroups') {
            // Disable all services that we selected via the servicegroup list
            $.post('ajaxreqs.php', { servicegroup: values[i], cmd: 'getservicesinservicegroup' }, function(data) {
                $.each(data, function(k, v) {
                    var name = v.host_name+' - '+v.service_description;
                    $('#tblServices .outputTableData:contains("'+name+'")').parents('tr').find('.remove a').trigger('click');
                    $('#selServices').find('option:contains("'+name+'")').attr('disabled', 'disabled').removeAttr('selected');
                });
            }, 'json');
        }
        
        // Input string for group or service selections
        var string = '<tr class="trOption" oldid="' + ids[i] + '" id="tr' + unique + '"><td><div class="outputTableData">' + titles[i] + '</div></td>';
        string += "<input class='hiddenList' type='hidden' name='" + postArray + "[]' value='" + values[i] + "' />";
        string += '<td>';

        // For template ordering
        if (postArray == 'hosttemplates' || postArray == 'servicetemplates') {
            string += '<div class="order-buttons"></div>';
        }

        string += '<div class="remove"><a  class="xBox" href="javascript:void(0)" onclick="remove_row(\'tr' + unique + '\', \'' + postArray + '\',\'' + values[i] + '\', \''+ titles[i] +'\');"></a></div></td></tr>';

        // Write output to new table/form
        var output = '#' + outputDiv;

        // Prevent duplicate entries
        var itemID = '#' + thisID;
        $(itemID).hide();
        $(itemID).attr('disabled', 'disabled');
        $(itemID).attr('selected', '');

        // Old dupliate prevention
        var duplicate = false;

        // If new value is not already there, insert it
        if (duplicate == false) {
            $(output+' tbody').append(string);
        }

        // For template ordering
        if (postArray == 'hosttemplates' || postArray == 'servicetemplates') {
            var upName = postArray.charAt(0).toUpperCase() + postArray.slice(1);
            if ($('#sel' + upName).has('option').length) {
                var tblName = "tbl" + upName;
                reorder_templates(tblName);
            }
        }
    }
}

// Removes item from output table and arrays of selected items
//
//  id - removes the <tr> by unique id
//  arrayType - tells which array to remove the selected item from
//  value - tells what value to erase from the selected array 
//
function remove_row(id, arrayType, value, name)
{
    var ID = '#' + id;
    var oldID = '#' + $(ID).attr('oldid');
    $(ID).remove();
    delete uniqueIDs[id];
    $(oldID).show();
    $(oldID).attr('disabled', false);
    $(oldID).attr('selected', 'selected');

    // If hostgroup or servicegroup
    if (arrayType == 'hostgroups') {
        $.post('ajaxreqs.php', { hostgroup: name, cmd: 'gethostsinhostgroup' }, function(data) {
            $.each(data, function(k, v) {
                $('#selHosts').find('option:contains("'+v+'")').attr('disabled', false);
            });
        }, 'json');
    } else if (arrayType == 'servicegroups') {
        $.post('ajaxreqs.php', { servicegroup: name, cmd: 'getservicesinservicegroup' }, function(data) {
            $.each(data, function(k, v) {
                var x = v.host_name+' - '+v.service_description;
                $('#selServices').find('option:contains("'+x+'")').attr('disabled', false);
            });
        }, 'json');
    }

}

function remove_all(tbl)
{
    if (tbl == 'tblHostgroups' || tbl == 'tblServicegroups') {
        $('#' + tbl + ' tr .remove a').trigger('click');
    } else {
        $('#' + tbl + ' tr.trOption').each(function () {
            var oldid = $(this).attr('oldid');
            $('#' + oldid).attr('disabled', false);
            $('#' + oldid).show();
            $(this).remove();
        });
    }
}

function remove(id) {
    $('#' + id).remove();
}

// Get all contact relationships
function getContactRelationships()
{
    var id = $('#contact').val();
    var contact = encodeURI($("#contact option:selected").text());
    url = 'ajaxreqs.php?cmd=getcontacts&contact=' + contact + '&id=' + id;
    $('#relationships').load(url);
}

// Get all contact group relationships
function getContactGroupRelationships()
{
    var id = $('#contactgroup').val();
    var contactgroup = encodeURI($("#contactgroup option:selected").text());
    url = 'ajaxreqs.php?cmd=getcontactgroups&contactgroup=' + contactgroup + '&id=' + id;
    $('#relationships').load(url);
}

function getHostgroupRelationships()
{
    var id = $('#hostgroup').val();
    var hostgroup = encodeURI($("#hostgroup option:selected").text());
    url = 'ajaxreqs.php?cmd=gethostgroups&hostgroup=' + hostgroup + '&id=' + id;
    $('#relationships').load(url);
}

function getParentHostRelationships()
{
    var id = $('#parenthost').val();
    var parenthost = encodeURI($("#parenthost option:selected").text());
    url = 'ajaxreqs.php?cmd=getparentshosts&parenthost=' + parenthost + '&id=' + id;
    $('#relationships').load(url);
}

var allChecked = false;
var allCheckedServices = false;
var allCheckedHosts = false

// Check all items on page, i = number of results on page  
function checkAll()
{
    if (allChecked) {
        $('input:checkbox').each(function () {
            this.checked = '';
        });
        $('#checkAll').text('Check All');
        allChecked = false;
    } else {
        $('input:checkbox').each(function () {
            this.checked = 'checked';
        });
        $('#checkAll').text('Deselect All');
        allChecked = true;
    }
}

// Bad hack for bulk modification tool 
function checkAllType(type)
{
    bool = (type == 'host') ? allCheckedHosts : allCheckedServices;
    if (bool) {
        $('input.' + type).each(function () {
            this.checked = '';
        });
        $('#checkAll' + type).text('Check All');
        if (type == 'host')
            allCheckedHosts = false;
        else
            allCheckedServices = false;
    } else {
        //var checked_status = this.checked;
        $('input.' + type).each(function () {
            this.checked = 'checked';
        });
        $('#checkAll' + type).text('Deselect All');
        if (type == 'host')
            allCheckedHosts = true;
        else
            allCheckedServices = true;
    }
}

// Reorder the templates
function reorder_templates(inputDiv)
{
    var stack = Array();
    stack = genereate_reordered_stack(inputDiv);

    display_reordered_templates(stack, inputDiv);
    bind_reorder_buttons(stack);
}

// Update the HTML for the reordered templates
function display_reordered_templates(stack, inputDiv)
{
    // Replace the stack in the HTML
    $.each(stack, function(k, v) {
        $('#' + inputDiv + ' tbody').append(v);
    });
}

function bind_reorder_buttons(stack)
{
    // Rebind the buttons
    $('.move').unbind('click');
    $('.move').click(function()
    {
        var inputDiv = $(this).parents('table.outputTable').attr('id');
        var id = $(this).parent().attr('stackid');

        if ($(this).hasClass('up')) {
            // Move the item up
            id = parseInt(id);
            up = id-1;
            var tmp = stack[id];
            var tmp2 = stack[up];
            stack[id] = tmp2;
            stack[up] = tmp;
        } else if ($(this).hasClass('down')) {
            // Move the item down
            id = parseInt(id);
            down = id+1;
            var tmp = stack[id];
            var tmp2 = stack[down];
            stack[id] = tmp2;
            stack[down] = tmp;
        }

        display_reordered_templates(stack, inputDiv);
        reorder_templates(inputDiv);
    });
}

function genereate_reordered_stack(inputDiv)
{
    var stack = [];

    // Pull the stack out of the HTML and add on buttons to change order
    var size = $("#" + inputDiv + " tr.trOption").size() - 1;
    $("#" + inputDiv + " tr.trOption").each(function(i)
    {
        // Remove all the elements
        $(this).find('.order-buttons').html('');
        $(this).find('.order-buttons').attr("stackid", i);

        // Append the buttons
        if (i == 0) {
            $(this).find(".order-buttons").append('<div class="move down"></div>');
        } else if (i < size) {
            $(this).find(".order-buttons").append('<div class="move up"></div><div class="move down"></div>');
        } else {
            $(this).find(".order-buttons").append('<div class="move up"></div>');
        }

        // Add to the new stack
        stack[i] = $(this).detach();
    });

    return stack;
}

// ---------------------------------------
//  Extra Helper Functions
// ---------------------------------------

// List filter function
jQuery.fn.filterByText = function(textbox, selectSingleMatch, minimumTextValue) {
    return this.each(function() {
        var select = this;
        var optionsAndOptGroups = [];

        if(typeof selectSingleMatch === "undefined") { selectSingleMatch = false; }
        if(typeof minimumTextValue === "undefined") { minimumTextValue = 0; }
        
        //Find all options and option groups
        $(select).children('option, optgroup').each(function() {
            optionsAndOptGroups.push($(this));
        });

        $(select).data('optionsAndOptGroups', optionsAndOptGroups);

        $(textbox).bind('keyup', function(e) {
            if(textbox.val().length > minimumTextValue || e.which === 46 || e.which === 8) {
                var optionsAndOptGroups = $(select).empty().data('optionsAndOptGroups');
                var search = $.trim($(this).val());
                var regex = new RegExp(search,'gi');

                $.each(optionsAndOptGroups, function() {
                    if($(this).is('option')) {
                        if($(this).text().match(regex) !== null) {
                            $(select).append($(this));
                        }
                    } else {
                        var optionGroupClone = $(this).clone(); //Clone the option
                        
                        $.each(optionGroupClone.children('option'), function() {
                            if($(this).text().match(regex) === null) {
                                $(this).remove();
                            }
                        });
                        
                        if(optionGroupClone.children().length) { $(select).append(optionGroupClone); }
                    }
                });
            }
        });

        if (selectSingleMatch === true && $(select).children().length === 1) {
            $(select).children().get(0).selected = true;
        }
    });
};